/**
 * @file  ATmega128.h
 * @brief Schnittstelle fuer die Klasse CATmega128.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _ATMEGA128_H_
#define _ATMEGA128_H_

#include "typedef.h"

// 128kByte Program - Speicher
#define ATMEGA128_SIZE_PROGRAM     0x20000
#define ATMEGA128_SIZE_BOOTLOADER  0x1000
#define ATMEGA128_SIZE_APPLICATION (ATMEGA128_SIZE_PROGRAM - ATMEGA128_SIZE_BOOTLOADER)

// 4kByte EEPROM - Speicher
#define ATMEGA128_SIZE_EEPROM  0x1000

class CATmega128
{
public:
	CATmega128();
	virtual ~CATmega128();

	// Ruecksetzen
	void Reset();

	// EEPROM
	inline UINT8 *GetMemEeprom(UINT32 address) { 
		if (address >= ATMEGA128_SIZE_EEPROM) 
		{ ASSERT(0); return 0; }
		return &m_eeprom[address]; 
	}
	inline UINT8 GetEeprom(UINT32 address) { 
		if (address >= ATMEGA128_SIZE_EEPROM) 
		{ ASSERT(0); return 0; }
		return m_eeprom[address]; 
	}
	inline UINT32 GetEepromRange(UINT32 address, UINT8 *value, UINT32 size) { 
		if ((address + size) >= ATMEGA128_SIZE_EEPROM)
		{ ASSERT(0); return 0; }
		UINT32 i;
		for (i=address; i < (address + size); i++) {
		
			value[i - address] = m_eeprom[i];
		}
		return (i - address);
	}
	inline void SetEeprom(UINT32 address, UINT8 value) { 
		if (address >= ATMEGA128_SIZE_EEPROM)
		{ ASSERT(0); return; }
		m_eeprom[address] = value; 
	}
	inline void SetEepromRange(UINT32 address, UINT8 *value, UINT32 size) { 
		if ((address + size) >= ATMEGA128_SIZE_EEPROM)
		{ ASSERT(0); return; }
		for (UINT32 i=address; i < (address + size); i++) {
		
			m_eeprom[i] = value[i - address];
			SetEepromUsebit(i);
		}
	}
	// PROGRAMFLASH
	inline UINT8 *GetMemProgram(UINT32 address) { 
		if (address >= ATMEGA128_SIZE_PROGRAM) 
		{ ASSERT(0); return 0; }
		return &m_program[address]; 
	}
	inline UINT8 GetProgram(UINT32 address) { 
		if (address >= ATMEGA128_SIZE_PROGRAM) 
		{ ASSERT(0); return 0; }
		return m_program[address]; 
	}
	inline UINT32 GetProgramRange(UINT32 address, UINT8 *value, UINT32 size) { 
		if ((address + size) >= ATMEGA128_SIZE_PROGRAM)
		{ ASSERT(0); return 0; }
		UINT32 i;
		for (i=address; i < (address + size); i++) {
		
			value[i - address] = m_program[i];
		}
		return (i - address);
	}
	inline void SetProgram(UINT32 address, UINT8 value) { 
		if (address >= ATMEGA128_SIZE_PROGRAM)
		{ ASSERT(0); return; }
		m_program[address] = value;
		SetProgramUsebit(address);
	}
	inline void SetProgramRange(UINT32 address, UINT8 *value, UINT32 size) { 
		if ((address + size) >= ATMEGA128_SIZE_PROGRAM)
		{ ASSERT(0); return; }
		for (UINT32 i=address; i < (address + size); i++) {
		
			m_program[i] = value[i - address];
			SetProgramUsebit(i);
		}
	}

	// Helper
	UINT8 GetEepromUsebit(UINT32 address);
	UINT8 GetProgramUsebit(UINT32 address);
	UINT32 GetEepromFirstAddress();
	UINT32 GetProgramFirstAddress();
	UINT32 GetEepromLastAddress();
	UINT32 GetProgramLastAddress();

protected:
	UINT8 m_eeprom[ATMEGA128_SIZE_EEPROM];
	UINT8 m_program[ATMEGA128_SIZE_PROGRAM];
	UINT32 m_eepromUse[ATMEGA128_SIZE_EEPROM / 32];
	UINT32 m_programUse[ATMEGA128_SIZE_PROGRAM / 32];
	
	// Helper
	void SetEepromUsebit(UINT32 address);
	void SetProgramUsebit(UINT32 address);
};

#endif

